@@ -7,16 +7,21 @@ combine.character <- function(x,y)
 
 ### combine lists by adding elements or adding to existing elements
 combine.list <- function(x,y){
-  toadd <- !names(y)%in%names(x)
-  toup <- names(y)[names(y)%in%names(x)]
+  toadd <- if(".overwrite"%in%names(y)){
+    y <- y[names(y)!=".overwrite"]
+    rep(TRUE,length(y))
+  }else{
+    !names(y)%in%names(x)
+  }
+  toup <- names(y)[!toadd]
   x[names(y)[toadd]] <- y[toadd]
   for(up in toup)x[[up]] <- combine(x[[up]],y[[up]])
-  return(x)
+  x
 ### A list, same type as x, but with added elements from y.
 }
 
 ### Prefix for code comments used with grep and gsub.
-prefix <- "^[ \t]*###[ \t]"
+prefix <- "^[ \t]*###[ \t]*"
 
 decomment <- function
 ### Remove comment prefix and join lines of code to form a
@@ -24,14 +29,14 @@ decomment <- function
 (comments
 ### Character vector of prefixed comment lines.
  ){
-  paste(gsub(prefix,"",comments),collapse="\n")
+  gsub(prefix,"",comments)
 ### String without prefixes or newlines.
 }
 
-### For each object in the package that satisfies the criterion#
-## checked by subfun, parse source using FUN and return the resulting
-### documentation list.
 forall <- function
+### For each object in the package that satisfies the criterion
+### checked by subfun, parse source using FUN and return the resulting
+### documentation list.
 (FUN,
 ### Function to apply to each element in the package.
  subfun=function(x)TRUE
@@ -43,11 +48,13 @@ forall <- function
   f <- function(objs,docs,...){
     objs <- objs[sapply(objs,subfun)]
     L <- list()
-    for(N in names(docs)){
+    on.exit(cat(sprintf("Parser Function failed on %s\n",N)))
+    for(N in union(names(docs),names(objs))){
       o <- objs[[N]]
       L[[N]] <- FUN(src=attr(o,"source"),
                     name=N,objs=objs,o=o,docs=docs,doc=docs[[N]],...)
     }
+    on.exit()## remove warning message
     L
   }
   class(f) <- c("allfun","function")
@@ -67,60 +74,22 @@ print.allfun <- function(x,...){
 ### For each function in the package, do something.
 forfun <- function(FUN)forall(FUN,is.function)
 
-examples.after.return <- function
-### Get examples from inline definitions after return()
-### PhG: this does not work well! Think of these situations:
-### 1) You have multiple return() in the code of your function,
-### 2) You have return() appearing is some example code, ...
-### I can hardly propose a hack here. The whole code of the function
-### must be parsed, and one must determine which one is the last line
-### of code that is actually executed.
-###
-### I make two propositions here
-### 1) to keep the same mechanism that has the advantage of simplicity
-###    but to use a special tag
-###    ##examples<< or #{{{examples to separate
-###    function code from examples explicitly, and
-### 2) to place the example in an "ex" attribute
-###    attached to the function
-###    (see next parser). That solution will be also interesting for
-###    documenting datasets, something not done yet by inlinedocs!
-(src,name="",...) { 
-  ## Look for the examples mark
-  m <- grep("^\\s*(##examples<<|#\\{\\{\\{examples)", src)
-  if (!length(m)) return(list())
-  if (length(m) > 1)
-    warning("More than one examples tag for ", name,
-            ". Taking the last one")
-  m <- m[length(m)]
-  ## Look for the lines containing return value comments just before
-  r <- grep("\\s*### ", src[1:(m-1)])
-    if (!length(r)) {
-      value <- NULL
-    } else {
-      ## Only take consecutive lines before the mark
-      keep <- rev((m - rev(r)) == 1:length(r))
-      if (!any(keep)) {
-        value <- NULL
-      } else {
-        value <- decomment(src[r[keep]])
-      }
-    }
-  ## Collect now the example code beneath the mark
-  ex <- src[(m + 1):(length(src) - 1)]
-  ## Possibly eliminate a #}}} tag
-  ex <- ex[!grepl("#}}}", ex)]
-  ## Eliminate leading tabulations or four spaces
-  prefixes <- gsub("(\\s*).*","\\1",ex,perl=TRUE)[grep("\\w",ex)]
+kill.prefix.whitespace <- function
+### Figure out what the whitespace preceding the example code is, and
+### then delete that from every line.
+(ex
+### character vector of example code lines.
+ ){
+  tlines <- gsub("\\s*","",ex)
+  ##tlines <- gsub("#.*","",tlines)
+  prefixes <- unique(gsub("\\S.*","",ex[tlines!=""]))
   FIND <- prefixes[which.min(nchar(prefixes))]
-  ex <- sub(FIND,"",ex)
-  ## Add an empty line before and after example
-  ex <- c("", ex, "")
-  ## Return examples and value
-  list(examples = paste(ex, collapse = "\n"), value = value)
+  ## Eliminate leading tabulations or 2/4 spaces
+  sub(FIND, "", ex)
+### Character vector of code lines with preceding whitespace removed.
 }
 
-prefixed.lines <- function(src,...){
+prefixed.lines <- structure(function(src,...){
 ### The primary mechanism of inline documentation is via consecutive
 ### groups of lines matching the specified prefix regular expression
 ### "\code{^### }" (i.e. lines beginning with "\code{### }") are
@@ -136,18 +105,20 @@ prefixed.lines <- function(src,...){
   starts <- c(1,bounds+1)
   ends <- c(bounds,length(clines))
   ## detect body of function using paren matching
-  f <- function(ch)cumsum(nchar(gsub(sprintf("[^%s]",ch),"",src)))
+  code <- gsub("#.*","",src)
+  f <- function(ch)cumsum(nchar(gsub(sprintf("[^%s]",ch),"",code)))
   parens <- f("(")-f(")")
   body.begin <- which(diff(parens)<0 & parens[-1]==0)+2
+  if(length(body.begin)==0)body.begin <- 1 ## rare cases
   is.arg <- function(){
-    0 == length(grep("^\\s*#",src[start-1],perl=TRUE)) &&
-      start<=body.begin
-    }
+    gres <- grep("^\\s*#",src[start-1],perl=TRUE)
+    0 == length(gres) && start<=body.begin
+  }
   res <- list()
   for(i in seq_along(starts)){
     start <- clines[starts[i]]
     end <- clines[ends[i]]
-    lab <- if(end+1==length(src))"value"
+    lab <- if(all(grepl("^\\s*#",src[end:(length(src)-1)])))"value"
     else if(start==2)"description"
     else if(is.arg()){
       ##twutz: strip leading white spaces and brackets and ,
@@ -163,232 +134,32 @@ prefixed.lines <- function(src,...){
     res[[lab]] <- decomment(src[start:end])
   }
   res
-}
-
-### Parsers for each object/function that are constructed
-### automatically. This is a named list, each element is a list of 2
-### elements: forfun/forall, then a parser function for an individual
-### object.
-forfun.parsers <-
-  list(prefixed.lines=prefixed.lines,
-       examples.after.return=examples.after.return,
-       ## Extract lots of info from normal functions.
-       parsefun=function(src,name,...){
-         extract.docs.fun(src,name)
-       },
-       ## title from first line of function def
-       title.from.firstline=function(src,name,code,...){
-         if(length(grep("#",src[1]))){
-           list(title=gsub("[^#]*#\\s*(.*)","\\1",src[1],perl=TRUE))
-         } else list()
-       },
-       ## PhG: it is tests/FUN.R!!! I would like more flexibility here
-       ## please, let me choose which dir to use for examples!
-       ## Get examples for FUN from the file tests/FUN.R
-       examples.from.testfile=function(name,...){
-         tsubdir <- getOption("inlinedocs.exdir")
-         if (is.null(tsubdir)) tsubdir <- "tests"	# Default value
-         tfile <- file.path("..",tsubdir,paste(name,".R",sep=""))
-         if(file.exists(tfile))
-           list(examples=paste(readLines(tfile),collapse="\n"))
-         else list()
-       },
-       definition.from.source=function(doc,src,...){
-         def <- doc$definition
-         is.empty <- function(x)is.null(x)||x==""
-         if(is.empty(def) && !is.empty(src))
-           list(definition=paste(src,collapse="\n"))
-         else list()
-       })
-
-### List of Parser Functions that can be applied to any object.
-forall.parsers <-
-  list(## Fill in author from DESCRIPTION and titles.
-       author.from.description=function(desc,...){
-         list(author=desc[,"Maintainer"])
-       },
-       ## The format section sometimes causes problems, so erase it.
-       erase.format=function(...){
-         list(format="")
-       },
-       ## Convert the function name to a title.
-       title.from.name=function(name,doc,...){
-         if("title"%in%names(doc))list() else
-         list(title=gsub("[._]"," ",name))
-       },
-       ## PhG: here is what I propose for examples code in the 'ex' attribute
-       examples.in.attr =  function (name, o, ...) {
-         ex <- attr(o, "ex")
-         if (!is.null(ex)) {
-           ## Special case for code contained in a function
-           if (inherits(ex, "function")) {
-             ## If source is available, start from there
-             src <- attr(ex, "source")
-             if (!is.null(src)) {
-               ex <- src
-             } else { ## Use the body of the function
-               ex <- deparse(body(ex))
-             }
-             ## Eliminate leading and trailing code
-             ex <- ex[-c(1, length(ex))]
-             ## Eliminate leading tabulations or 2/4 spaces
-             ex <- sub("^\t|    |  ", "", ex)
-             ## Add an empty line before and after example
-             ex <- c("", ex, "")
-           }
-           list(examples = paste(ex, collapse = "\n"))
-         } else list()
-       })
-
-### List of parser functions that operate on single objects. This list
-### is useful for testing these functions.
-lonely <- c(forall.parsers,forfun.parsers)
-attr(lonely,"ex") <- function(){
-  lonely$parsefun(attr(extract.docs.file,"source"),"extract.docs.file")
-}
-
-extra.code.docs <- function # Extract documentation from code chunks
-### Parse R code to extract inline documentation from comments around
-### each function. These are not able to be retreived simply by
-### looking at the "source" attribute. This is a Parser Function that
-### can be used in the parser list of package.skeleton.dx(). TODO:
-### Modularize this into separate Parsers Functions for S4 classes,
-### prefixes, ##<<blocks, etc. Right now it is not very clean!
-(code,
-### Code lines in a character vector containing multiple R objects to
-### parse for documentation.
- objs,
-### The objects defined in the code.
- ...
-### ignored
+},ex=function(){
+test <- function
+### the description
+(x,
+### the first argument
+ y ##<< another argument
  ){
-  parsed <- extract.file.parse(code)
-  extract.docs.try <- function(o,on)
-    {
-      ## Note: we could use parsed information here too, but that
-      ## would produce different results for setMethodS3 etc.
-      doc <- list()
-      if ( !is.null(parsed[[on]]) ){
-        if ( !is.na(parsed[[on]]@code[1]) ){ # no code given for generics
-          doc$definition <- paste(parsed[[on]]@code,collapse="\n")
-        }
-        if(!"description"%in%names(doc) && !is.na(parsed[[on]]@description) ){
-          doc$description <- parsed[[on]]@description
-        }
-        if ( "setMethodS3" == parsed[[on]]@created ){
-          # PhG: this may be wrong! It does not catch correctly how the method
-	  # must be splitted in case of methods containing dots. for instance,
-	  # as.data.frame.matrix must be split into: m1 = as.data.frame and
-	  # m2 = matrix... here you got m1 = as, and m2 = data.frame.matrix!!!
-		  pattern <- "^([^\\.]+)\\.(.*)$"
-          doc$s3method=c(m1 <- gsub(pattern,"\\1",on,perl=TRUE),
-              m2 <- gsub(pattern,"\\2",on,perl=TRUE))
-          if ( 0 < length(grep("\\W",m1,perl=TRUE)) ){
-			  m1 <- paste("`",m1,"`",sep="")
-          }
-          cat("S3method(",m1,",",m2,")\n",sep="")
-        }
-      }
-      if("title" %in% names(doc) && !"description" %in% names(doc) ){
-        ## For short functions having both would duplicate, but a
-        ## description is required. Therefore automatically copy title
-        ## across to avoid errors at package build time.
-        doc$description <- doc$title
-      }
-      doc
-    }
-  extract.docs <- function(on){
-    res <- try({o <- objs[[on]]
-                extract.docs.try(o, on)},FALSE)
-    if(class(res)=="try-error"){
-      cat("Failed to extract docs for: ",on,"\n\n")
-      list()
-    } else if(0 == length(res) && inherits(objs[[on]],"standardGeneric")){
-      NULL
-    } else if(0 == length(res) && "function" %in% class(o)
-              && 1 == length(osource <- attr(o,"source"))
-              && 1 == length(grep(paste("UseMethod(",on,")",sep="\""),osource))
-              ){
-      ## phew - this should only pick up R.oo S3 generic definitions like:
-      ## attr(*, "source")= chr "function(...) UseMethod(\"select\")"
-      NULL
-    } else res
-  }
-  doc.names <- names(objs)
-  res <- sapply(doc.names,extract.docs,simplify=FALSE)
-  ## Special processing for S4 classes as they do not appear in normal ls()
-  for ( nn in names(parsed) ){
-    if ( parsed[[nn]]@created == "setClass" ){
-      S4class.docs <- extract.docs.setClass(parsed[[nn]])
-      docname <- paste(nn,"class",sep="-")
-      if ( is.null(res[[docname]]) ){
-        res[[docname]] <- S4class.docs
-        doc.names <- c(doc.names,docname)
-      } else {
-        stop(nn," appears as both S4 class and some other definition")
-      }
-    }
-  }
-  inherit.docs <- function(on){
-    in.res <- res[[on]]
-    if ( !is.null(parsed[[on]]) ){
-      for ( parent in parsed[[on]]@parent ){
-        if ( !is.na(parent) ){
-          if ( is.null(in.res) ){
-            in.res <- res[[parent]]
-          } else if ( parent %in% names(res) ){
-            parent.docs <- res[[parent]]
-            for ( nn in names(parent.docs) ){
-              if ( !nn %in% names(in.res) ){
-                in.res[[nn]] <- parent.docs[[nn]]
-              }
-            }
-          }
-        }
-      }
-    }
-    invisible(in.res)
-  }
-  all.done <- FALSE
-  while ( !all.done ){
-    res1 <- sapply(doc.names,inherit.docs,simplify=FALSE)
-    all.done <- identical(res1,res)
-    res <- res1
-  }
-  ## now strip out any generics (which have value NULL in res):
-  res.not.null <- sapply(res,function(x){!is.null(x)})
-  if ( 0 < length(res.not.null) && length(res.not.null) < length(res) ){
-    res <- res[res.not.null]
-  }
-  res
-### named list of lists, one for each object to document.
+  5
+### the return value
+##seealso<< foobar
 }
+src <- attr(test,"source")
+prefixed.lines(src)
+extract.xxx.chunks(src)
+})
 
-### List of parsers to use by default with package.skeleton.dx.
-default.parsers <-
-  c(extra.code.docs=extra.code.docs, ## TODO: cleanup!
-    sapply(forfun.parsers,forfun),
-    sapply(forall.parsers,forall),
-    edit.package.file=function(desc,...){
-      in.details <- setdiff(colnames(desc),"Description")
-      details <- paste(paste(in.details,": \\tab ",desc[,in.details],"\\cr",
-                             sep=""),collapse="\n")
-      L <-
-        list(list(title=desc[,"Title"],
-                  description=desc[,"Description"],
-                  `tabular{ll}`=details,
-                  author=desc[,"Maintainer"]))
-      names(L) <- paste(desc[,"Package"],"-package",sep="")
-      L
-    })
-
-extract.docs.fun <- function # Extract documentation from a function
+extract.xxx.chunks <- function # Extract documentation from a function
 ### Given source code of a function, return a list describing inline
 ### documentation in that source code.
-(code,
-### The function to examine.
- name.fun
+(src,
+### The source lines of the function to examine, as a character
+### vector.
+ name.fun="(unnamed function)",
 ### The name of the function/chunk to use in warning messages.
+ ...
+### ignored.
  ){
   res <- list()
   ##details<< For simple functions/arguments, the argument may also be
@@ -463,15 +234,15 @@ extract.docs.fun <- function # Extract documentation from a function
         chunk.sep <- "\n\n"
       }
       chunk.res <- NULL
-      if ( 0 == length(grep("^\\s*$",payload,perl=TRUE)) )
+      if ( !grepl("^\\s*$",payload,perl=TRUE) )
         chunk.res <-
           if ( is.null(res[[field]]) ) payload
           else paste(res[[field]], payload, sep=chunk.sep)
       invisible(chunk.res)
     }
-  while ( k <= length(code) ){
-    line <- code[k]
-    if ( 0 < length(grep(extra.regexp,line,perl=TRUE) ) ){
+  while ( k <= length(src) ){
+    line <- src[k]
+    if ( grepl(extra.regexp,line,perl=TRUE) ){
       ## we have a new extra chunk - first get field name and any payload
       new.field <- gsub(extra.regexp,"\\1",line,perl=TRUE)
       new.contents <- gsub(extra.regexp,"\\2",line,perl=TRUE)
@@ -546,9 +317,9 @@ extract.docs.fun <- function # Extract documentation from a function
           first.describe <- TRUE;
         }
       }
-    } else if ( in.chunk && 0<length(grep(cont.re,line,perl=TRUE)) ){
+    } else if ( in.chunk && grepl(cont.re,line,perl=TRUE) ){
       ## append this line to current chunk
-      if ( 0 == length(grep(prefix,line,perl=TRUE)) ){
+      if ( !grepl(prefix,line,perl=TRUE) ){
         ##describe<< Any lines with "\code{### }" at the left hand
         ## margin within the included chunks are handled separately,
         ## so if they appear in the documentation they will appear
@@ -561,7 +332,7 @@ extract.docs.fun <- function # Extract documentation from a function
           payload <- stripped
         }
       }
-    } else if ( 0 < length(grep(arg.pat,line,perl=TRUE)) ){
+    } else if ( grepl(arg.pat,line,perl=TRUE) ){
       not.describe <- (0==in.describe && !first.describe)
       if ( in.chunk && not.describe){
         res[[cur.field]] <- end.chunk(cur.field,payload)
@@ -623,10 +394,269 @@ extract.docs.fun <- function # Extract documentation from a function
 ### with the string in this list (implemented in modify.Rd.file).
 }
 
+leadingS3generic <- function # check whether function name is an S3 generic
+### Determines whether a function name looks like an S3 generic function
+(name,                     ##<< name of function
+ env,                      ##<< environment to search for additional generics
+ ...)                      ##<< ignored here
+{
+  ##details<< This function is one of the default parsers, but exposed as
+  ## possibly of more general interest. Given a function name of the form
+  ## x.y.z it looks for the generic function x applying to objects of class
+  ## y.z and also for generic function x.y applying to objects of class z.
+  ##
+  parts <- strsplit(name, ".", fixed = TRUE)[[1]]
+  l <- length(parts)
+  if (l > 1) {
+    for (i in 1:(l - 1)) {
+      ## Look for a generic function (known by the system or defined
+      ## in the package) that matches that part of the function name
+      generic <- paste(parts[1:i], collapse = ".")
+      if (any(generic %in% utils:::getKnownS3generics()) ||
+          utils:::findGeneric(generic, env) != "") {
+        object <- paste(parts[(i + 1):l], collapse = ".") 
+        ##details<< Assumes that the first name which matches any known
+        ## generics is the target generic function, so if both x and x.y
+        ## are generic functions, will assume generic x applying to objects
+        ## of class y.z
+        ##value<< If a matching generic found returns a list with a single component:
+        return(list(.s3method=c(generic, object))) ##<< a character vector containing generic name and object name.
+      }
+    }
+  }
+  ##value<< If no matching generic functions are found, returns an empty list.
+  list()
+}
+
+### Parsers for each function that are constructed automatically. This
+### is a named list, and each element is a parser function for an
+### individual object.
+forfun.parsers <-
+  list(prefixed.lines=prefixed.lines,
+       extract.xxx.chunks=extract.xxx.chunks,
+       ## title from first line of function def
+       title.from.firstline=function(src,...){
+         first <- src[1]
+         if(!is.character(first))return(list())
+         if(!grepl("#",first))return(list())
+         list(title=gsub("[^#]*#\\s*(.*)","\\1",first,perl=TRUE))
+       },
+       ## PhG: it is tests/FUN.R!!! I would like more flexibility here
+       ## please, let me choose which dir to use for examples!
+       ## Get examples for FUN from the file tests/FUN.R
+       examples.from.testfile=function(name,...){
+         tsubdir <- getOption("inlinedocs.exdir")
+         if (is.null(tsubdir)) tsubdir <- "tests"	# Default value
+         tfile <- file.path("..",tsubdir,paste(name,".R",sep=""))
+         if(file.exists(tfile))
+           list(examples=readLines(tfile))
+         else list()
+       },
+       definition.from.source=function(doc,src,...){
+         def <- doc$definition
+         is.empty <- function(x)is.null(x)||x==""
+         if(is.empty(def) && !is.empty(src))
+           list(definition=src)
+         else list()
+       })
+
+### List of Parser Functions that can be applied to any object.
+forall.parsers <-
+  list(## Fill in author from DESCRIPTION and titles.
+       author.from.description=function(desc,...){
+         list(author=desc[,"Maintainer"])
+       },
+       ## The format section sometimes causes problems, so erase it.
+       erase.format=function(...){
+         list(format="")
+       },
+       ## Convert the function name to a title.
+       title.from.name=function(name,doc,...){
+         if("title"%in%names(doc))list() else
+         list(title=gsub("[._]"," ",name))
+       },
+       ## PhG: here is what I propose for examples code in the 'ex' attribute
+       examples.in.attr =  function (name, o, ...) {
+         ex <- attr(o, "ex")
+         if (!is.null(ex)) {
+           ## Special case for code contained in a function
+           if (inherits(ex, "function")) {
+             ## If source is available, start from there
+             src <- attr(ex, "source")
+             if (!is.null(src)) {
+               ex <- src
+             } else { ## Use the body of the function
+               ex <- deparse(body(ex))
+             }
+             ## Eliminate leading and trailing code
+             ex <- ex[-c(1, length(ex))]
+             ## all the prefixes
+             ex <- kill.prefix.whitespace(ex)
+             ## Add an empty line before and after example
+             ex <- c("", ex, "")
+           }
+           list(examples = ex)
+         } else list()
+       },collapse=function(doc,...){
+         L <- lapply(doc,paste,collapse="\n")
+         L$.overwrite <- TRUE
+         L
+       },tag.s3methods=leadingS3generic
+       )
+
+### List of parser functions that operate on single objects. This list
+### is useful for testing these functions.
+lonely <- structure(c(forall.parsers,forfun.parsers),ex=function(){
+  f <- function # title
+### description
+  (x, ##<< arg x
+   y
+### arg y
+   ){
+    ##value<< a list with elements
+    list(x=x, ##<< original x value
+         y=y, ##<< original y value
+         sum=x+y) ##<< their sum
+    ##end<<
+  }
+  src <- attr(f,"source")
+  lonely$extract.xxx.chunks(src)
+  lonely$prefixed.lines(src)
+})
+
+extra.code.docs <- function # Extract documentation from code chunks
+### Parse R code to extract inline documentation from comments around
+### each function. These are not able to be retreived simply by
+### looking at the "source" attribute. This is a Parser Function that
+### can be used in the parser list of package.skeleton.dx(). TODO:
+### Modularize this into separate Parsers Functions for S4 classes,
+### prefixes, ##<<blocks, etc. Right now it is not very clean!
+(code,
+### Code lines in a character vector containing multiple R objects to
+### parse for documentation.
+ objs,
+### The objects defined in the code.
+ ...
+### ignored
+ ){
+  parsed <- extract.file.parse(code)
+  extract.docs.try <- function(o,on)
+    {
+      ## Note: we could use parsed information here too, but that
+      ## would produce different results for setMethodS3 etc.
+      doc <- list()
+      if ( !is.null(parsed[[on]]) ){
+        if ( !is.na(parsed[[on]]@code[1]) ){ # no code given for generics
+          doc$definition <- paste(parsed[[on]]@code)
+        }
+        if(!"description"%in%names(doc) && !is.na(parsed[[on]]@description) ){
+          doc$description <- parsed[[on]]@description
+        }
+        ## if ( "setMethodS3" == parsed[[on]]@created ){
+        ##   gen <- leadingS3generic(on,topenv())
+        ##   if ( 0 < length(gen) ){
+        ##     doc$.s3method <- gen$.s3method
+        ##     cat("S3method(",gen$.s3method[1],",",gen$.s3method[2],")\n",sep="")
+        ##   }
+        ## }
+      }
+      if("title" %in% names(doc) && !"description" %in% names(doc) ){
+        ## For short functions having both would duplicate, but a
+        ## description is required. Therefore automatically copy title
+        ## across to avoid errors at package build time.
+        doc$description <- doc$title
+      }
+      doc
+    }
+  extract.docs <- function(on){
+    res <- try({o <- objs[[on]]
+                extract.docs.try(o, on)},FALSE)
+    if(class(res)=="try-error"){
+      cat("Failed to extract docs for: ",on,"\n\n")
+      list()
+    } else if(0 == length(res) && inherits(objs[[on]],"standardGeneric")){
+      NULL
+    } else if(0 == length(res) && "function" %in% class(o)
+              && 1 == length(osource <- attr(o,"source"))
+              && grepl(paste("UseMethod(",on,")",sep="\""),osource)
+              ){
+      ## phew - this should only pick up R.oo S3 generic definitions like:
+      ## attr(*, "source")= chr "function(...) UseMethod(\"select\")"
+      NULL
+    } else res
+  }
+  doc.names <- names(objs)
+  res <- sapply(doc.names,extract.docs,simplify=FALSE)
+  ## Special processing for S4 classes as they do not appear in normal ls()
+  for ( nn in names(parsed) ){
+    if ( parsed[[nn]]@created == "setClass" ){
+      S4class.docs <- extract.docs.setClass(parsed[[nn]])
+      docname <- paste(nn,"class",sep="-")
+      if ( is.null(res[[docname]]) ){
+        res[[docname]] <- S4class.docs
+        doc.names <- c(doc.names,docname)
+      } else {
+        stop(nn," appears as both S4 class and some other definition")
+      }
+    }
+  }
+  inherit.docs <- function(on){
+    in.res <- res[[on]]
+    if ( !is.null(parsed[[on]]) ){
+      for ( parent in parsed[[on]]@parent ){
+        if ( !is.na(parent) ){
+          if ( is.null(in.res) ){
+            in.res <- res[[parent]]
+          } else if ( parent %in% names(res) ){
+            parent.docs <- res[[parent]]
+            for ( nn in names(parent.docs) ){
+              if ( !nn %in% names(in.res) ){
+                in.res[[nn]] <- parent.docs[[nn]]
+              }
+            }
+          }
+        }
+      }
+    }
+    invisible(in.res)
+  }
+  all.done <- FALSE
+  while ( !all.done ){
+    res1 <- sapply(doc.names,inherit.docs,simplify=FALSE)
+    all.done <- identical(res1,res)
+    res <- res1
+  }
+  ## now strip out any generics (which have value NULL in res):
+  res.not.null <- sapply(res,function(x){!is.null(x)})
+  if ( 0 < length(res.not.null) && length(res.not.null) < length(res) ){
+    res <- res[res.not.null]
+  }
+  res
+### named list of lists, one for each object to document.
+}
+
+### List of parsers to use by default with package.skeleton.dx.
+default.parsers <-
+  c(extra.code.docs=extra.code.docs, ## TODO: cleanup!
+    sapply(forfun.parsers,forfun),
+    edit.package.file=function(desc,...){
+      in.details <- setdiff(colnames(desc),"Description")
+      details <- sprintf("%s: \\tab %s\\cr",in.details,desc[,in.details])
+      L <-
+        list(list(title=desc[,"Title"],
+                  description=desc[,"Description"],
+                  `tabular{ll}`=details))
+      names(L) <- paste(desc[,"Package"],"-package",sep="")
+      L
+    },
+    sapply(forall.parsers,forall)
+    )
+
 setClass("DocLink", # Link documentation among related functions
 ### The \code{.DocLink} class provides the basis for hooking together
 ### documentation of related classes/functions/objects. The aim is that
-### documentation sections missing from the child are
+### documentation sections missing from the child are inherited from
+### the parent class.
          representation(name="character", ##<< name of object
                         created="character", ##<< how created
                         parent="character", ##<< parent class or NA
@@ -661,7 +691,7 @@ extract.file.parse <- function # File content analysis
     ## "prefix" lines will be used instead.
     default.description <- NULL
     while ( start > last.end+1
-           && 1 == length(grep(prefix,code[start-1],perl=TRUE)) ){
+           && grepl(prefix,code[start-1],perl=TRUE) ){
       start <- start-1
     }
     if ( start < chunks[[k]][1] ){
@@ -688,7 +718,7 @@ extract.file.parse <- function # File content analysis
       ## If the function definition is not embedded within the call, then
       ## the parent is that function. Test whether the the third value
       ## looks like a name and add it to parents if so.
-      if ( 1 == length(grep("^[\\._\\w]+$",chars[3],perl=TRUE)) ){
+      if ( grepl("^[\\._\\w]+$",chars[3],perl=TRUE) ){
         parent <- chars[3]
       }
       res[[object.name]] <- new("DocLink",name=object.name,
@@ -720,7 +750,7 @@ extract.file.parse <- function # File content analysis
       ## If the function definition is not embedded within the call, then
       ## the parent is that function. Test whether the the fourth value
       ## looks like a name and add it to parents if so.
-      if ( 1 == length(grep("^[\\._\\w]+$",chars[4],perl=TRUE)) ){
+      if ( grepl("^[\\._\\w]+$",chars[4],perl=TRUE) ){
         parent <- c(chars[4],parent)
       }
       res[[object.name]] <- new("DocLink",name=object.name,
@@ -761,7 +791,7 @@ extract.docs.setClass <- function # S4 class inline documentation
   ## the same line or \code{### } comments at the beginning of the
   ## following line.
   f.n <- paste(class.name,"class",sep="-")
-  docs <- extract.docs.fun(chunk.source,f.n)
+  docs <- extract.xxx.chunks(chunk.source,f.n)
   ## also apply source parsing functions that I separated out into
   ## separate functions
   docs <- combine(docs,lonely$prefixed.lines(chunk.source))
@@ -803,32 +833,33 @@ apply.parsers <- function
 ### Additional arguments to pass to Parser Functions.
  ){
   e <- new.env()
-  old <- options(keep.source=TRUE)
+  ## KMP 2011-03-09 fix problem with DocLink when inlinedocs ran on itself
+  ## Error in assignClassDef(Class, classDef, where) :
+  ##   Class "DocLink" has a locked definition in package "inlinedocs"
+  ## Traced to "where" argument in setClassDef which defaults to topenv()
+  ## which in turn is inlinedocs when processing inlinedocs package, hence
+  ## the clash. The following works (under R 2.12.2), so that the topenv()
+  ## now finds e before finding the inlinedocs environment.
+  old <- options(keep.source=TRUE,topLevelEnvironment=e)
   on.exit(options(old))
   exprs <- parse(text=code)
-  for (i in exprs) eval(i, e)
+  ## TDH 2011-04-07 set this so that no warnings about creating a fake
+  ## package when we try to process S4 classes defined in code
+  e$.packageName <- "inlinedocs.processor"
+  for (i in exprs){
+    ## TDH 2011-04-07 Disable this tryCatch since it makes it harder
+    ##to debug errors/warnings in the evaluated code
+
+    ##tryCatch({
+      eval(i, e)
+    ##},error=function(e){
+      ##print(e)
+    ##})
+  }
   objs <- sapply(ls(e),get,e,simplify=FALSE)
 
   docs <- list()
   
-  # PhG: Automatically determine who is S3 methods
-  for (name in names(objs)) {
-    parts <- strsplit(name, ".", fixed = TRUE)[[1]]
-	l <- length(parts)
-	if (l > 1) {
-        for (i in 1:(l - 1)) {
-            # Look for a generic function (known by the system or defined
-            # in the package) that matches that part of the function name
-            generic <- paste(parts[1:i], collapse = ".")
-            if (any(generic %in% utils:::getKnownS3generics()) ||
-                utils:::findGeneric(generic, e) != "") {
-                object <- paste(parts[(i + 1):l], collapse = ".") 
-                docs[[name]]$s3method <- c(generic, object)
-                break
-            }
-        }
-    }
-  }
   ## apply parsers in sequence to code and objs
   for(i in seq_along(parsers)){
     N <- names(parsers[i])
@@ -839,9 +870,16 @@ apply.parsers <- function
     }
     p <- parsers[[i]]
     ## This is the argument list that each parser receives:
-    L <- p(code=code,objs=objs,docs=docs,...)
+    L <- p(code=code,objs=objs,docs=docs,env=e,...)
     docs <- combine(docs,L)
   }
+  ## post-process to collapse all character vectors
+  for(i in seq_along(docs)){
+    for(j in seq_along(docs[[i]])){
+      if(names(docs[[i]])[j]!=".s3method")
+      docs[[i]][[j]] <- paste(docs[[i]][[j]],collapse="\n")
+    }
+  }
   if(verbose)cat("\n")
   docs
 ### A list of extracted documentation from code.
@@ -858,15 +896,20 @@ non.descfile.names <-
 ### description file.
 nondesc.parsers <- default.parsers[non.descfile.names]
 
-extract.docs.file <- function
+extract.docs.file <- structure(function
 ### Apply all parsers relevant to extract info from just 1 code file.
 (f,
 ### File name of R code to read and parse.
- parsers=nondesc.parsers,
+ parsers=NULL,
 ### Parser Functions to use to parse the code and extract
 ### documentation.
  ...
 ### Other arguments to pass to Parser Functions.
  ){
+  if(is.null(parsers))parsers <- nondesc.parsers
   apply.parsers(readLines(f),parsers,verbose=FALSE,...)
-}
+},ex=function(){
+  f <- system.file("silly","R","silly.R",package="inlinedocs")
+  extract.docs.file(f)
+})
+
